import React from 'react';
import ReactDOM from 'react-dom/client';
import App from './App';
import './index.css';
import { initializeDatabaseStructures } from './integrations/supabase/client';
import { toast } from 'sonner';

console.log('[DEBUG] EventSpace - App initialization starting in debug mode');

// Enhanced network error tracking
let networkErrorCount = 0;
const MAX_NETWORK_ERRORS = 3;
const NETWORK_ERROR_RESET_INTERVAL = 60000; // 1 minute

// Track app load time
const APP_LOAD_START_TIME = Date.now();
const MAX_APP_LOAD_TIME = 30000; // 30 seconds

// Add custom CSS to hide browser error messages
const addBrowserErrorBlockingStyles = () => {
  // Create a style element
  const style = document.createElement('style');
  style.type = 'text/css';
  
  // CSS to hide the browser error messages
  const css = `
    /* Target browser error messages */
    div[role="alert"] {
      display: none !important;
      visibility: hidden !important;
      opacity: 0 !important;
      pointer-events: none !important;
      position: absolute !important;
      z-index: -9999 !important;
    }
    
    /* Target refresh messages specifically */
    div[role="alert"]:has(button:contains("Refresh")) {
      display: none !important;
    }
    
    /* Target error icons */
    div[role="alert"] .error-icon {
      display: none !important;
    }
    
    /* Target toast with refresh message */
    div[role="region"][aria-label*="Notifications"] div:contains("Loading is taking longer") {
      display: none !important;
    }
    
    /* Hide all red background toasts with refresh buttons */
    div[class*="toast"][class*="error"],
    div[class*="toast"][class*="destructive"],
    div[class*="toast"][class*="red"] {
      display: none !important;
    }
    
    /* Hide all loading spinner timeouts */
    .loading-timeout-message {
      display: none !important;
    }
    
    /* Hide browser error banners */
    div[class*="banner"][class*="error"],
    .browser-error-banner,
    .timeout-banner {
      display: none !important;
    }
  `;
  
  // Add the CSS to the style element
  style.appendChild(document.createTextNode(css));
  
  // Add the style element to the head
  document.head.appendChild(style);
  
  console.log('[DEBUG] Added CSS to hide browser error messages');
};

// Add tab visibility state preservation to prevent refreshes when switching tabs
const setupStatePreservation = () => {
  let documentHidden = false;
  let lastFocusTime = Date.now();
  let lastHiddenTime = 0;
  
  // Add the custom CSS to hide browser error messages
  try {
    addBrowserErrorBlockingStyles();
  } catch (e) {
    console.error('[DEBUG] Error adding browser error blocking styles:', e);
  }
  
  // Track the document visibility to prevent refreshes when returning to tab
  document.addEventListener('visibilitychange', (event) => {
    console.log(`[DEBUG] Visibility changed: ${document.visibilityState}`);
    
    if (document.visibilityState === 'hidden') {
      documentHidden = true;
      lastHiddenTime = Date.now();
      // Store timestamp when tab became hidden
      sessionStorage.setItem('tab_hidden_time', Date.now().toString());
    } else if (document.visibilityState === 'visible' && documentHidden) {
      documentHidden = false;
      lastFocusTime = Date.now();
      console.log('[DEBUG] Tab is now visible again - preventing refresh');
      
      // Explicitly prevent any automatic refresh
      if (event) {
        try {
          event.preventDefault();
          event.stopPropagation();
        } catch (e) {
          console.error('[DEBUG] Error preventing default event:', e);
        }
      }
      
      // Calculate how long the tab was hidden
      const hiddenTime = sessionStorage.getItem('tab_hidden_time');
      if (hiddenTime) {
        const hiddenDuration = Date.now() - parseInt(hiddenTime);
        console.log(`[DEBUG] Tab was hidden for ${hiddenDuration}ms`);
        
        // Add a flag to prevent any subsequent handlers from refreshing the page
        window.sessionStorage.setItem('prevent_refresh', 'true');
        
        // Set a flag to indicate the tab just became visible again
        // This can be used by components to avoid unnecessary reloads
        window.sessionStorage.setItem('tab_just_activated', 'true');
        
        // Clear the flags after a short delay
        setTimeout(() => {
          window.sessionStorage.removeItem('prevent_refresh');
          window.sessionStorage.removeItem('tab_just_activated');
        }, 2000);
      }
      
      // Dismiss any browser timeout errors that might be showing
      dismissBrowserTimeoutErrors();
    }
  }, { capture: true }); // Use capture phase to intercept events early
  
  // Also handle the focus event
  window.addEventListener('focus', (event) => {
    console.log('[DEBUG] Window focus event captured');
    
    // Check if we're returning from being hidden
    const hiddenTime = sessionStorage.getItem('tab_hidden_time');
    const timeSinceLastFocus = Date.now() - lastFocusTime;
    
    if (hiddenTime || timeSinceLastFocus > 1000) {
      console.log('[DEBUG] Focus after tab was hidden - preventing refresh');
      lastFocusTime = Date.now();
      
      // Explicitly prevent any automatic refresh
      try {
        event.preventDefault();
        event.stopPropagation();
      } catch (e) {
        console.error('[DEBUG] Error preventing default focus event:', e);
      }
      
      // Set a flag to indicate the tab just received focus
      window.sessionStorage.setItem('tab_just_focused', 'true');
      setTimeout(() => {
        window.sessionStorage.removeItem('tab_just_focused');
      }, 2000);
      
      // Clear hidden time
      sessionStorage.removeItem('tab_hidden_time');
    }
  }, { capture: true });
  
  // Override the browser's error handling
  window.onerror = function(message, source, lineno, colno, error) {
    console.log('[DEBUG] Caught window error:', message);
    
    // Check if it's a timeout-related error
    if (message && typeof message === 'string' && 
        (message.includes('timeout') || message.includes('load') || message.includes('long'))) {
      console.log('[DEBUG] Suppressing timeout-related error message');
      return true; // Prevents the browser from showing its error UI
    }
    
    return false; // Let other errors propagate normally
  };
  
  // Also handle the beforeunload event to prevent accidental refreshes
  window.addEventListener('beforeunload', (event) => {
    // Only prevent unload if the page wasn't explicitly being navigated away from
    const isIntentionalNavigation = sessionStorage.getItem('intentional_navigation') === 'true';
    const preventRefresh = sessionStorage.getItem('prevent_refresh') === 'true';
    
    if (preventRefresh || (!isIntentionalNavigation && (Date.now() - lastFocusTime < 1000 || Date.now() - lastHiddenTime < 1000))) {
      // Prevent accidental refresh right after tab switching
      console.log('[DEBUG] Preventing potential accidental refresh after tab switch');
      event.preventDefault();
      // For modern browsers
      event.returnValue = '';
      return '';
    }
    
    // Check if we're on pages we want to protect from accidental navigation
    const onAddSpacePage = window.location.pathname.includes('add-space');
    const onEditSpacePage = window.location.pathname.includes('edit-space');
    
    if (!isIntentionalNavigation && (onAddSpacePage || onEditSpacePage)) {
      // Prevent accidental refresh/navigation when on important form pages
      console.log('[DEBUG] Preventing accidental navigation from form page');
      event.preventDefault();
      // For modern browsers
      event.returnValue = '';
      return '';
    }
    
    // Clear the intentional navigation flag
    sessionStorage.removeItem('intentional_navigation');
  });
  
  // Function to automatically dismiss browser timeout warnings
  const dismissBrowserTimeoutErrors = () => {
    console.log('[DEBUG] Setting up browser timeout error dismissal');
    
    // Immediately hide any existing error messages using CSS
    const styleTag = document.createElement('style');
    styleTag.textContent = `
      div[role="alert"] {
        display: none !important;
        visibility: hidden !important;
        opacity: 0 !important;
      }
      /* Hide any loading timeout messages */
      [data-loading-timeout="true"],
      .loading-timeout-container {
        display: none !important;
      }
    `;
    document.head.appendChild(styleTag);
    
    // Look for browser-level error messages about loading timeouts
    const timeoutElements = document.querySelectorAll('div[role="alert"]');
    timeoutElements.forEach(element => {
      console.log('[DEBUG] Found alert element, hiding it');
      if (element instanceof HTMLElement) {
        element.style.display = 'none';
        element.style.visibility = 'hidden';
        element.style.opacity = '0';
        element.style.pointerEvents = 'none';
      }
      
      const text = element.textContent || '';
      if (text.includes('Loading is taking longer') || 
          text.includes('taking longer than expected') || 
          text.includes('Refresh')) {
        console.log('[DEBUG] Found browser timeout warning, attempting to dismiss');
        try {
          // Try to find and click the close button if it exists
          const closeButton = element.querySelector('button');
          if (closeButton) {
            closeButton.click();
            console.log('[DEBUG] Automatically dismissed browser timeout warning');
          }
          
          // Also try removing the element entirely
          if (element && element.parentNode) {
            try {
              element.parentNode.removeChild(element);
              console.log('[DEBUG] Removed browser timeout warning from DOM');
            } catch (removeError) {
              console.error('[DEBUG] Failed to remove element:', removeError);
              // Fall back to hiding the element if removal fails
              if (element instanceof HTMLElement) {
                element.style.display = 'none';
                element.style.opacity = '0';
                element.style.pointerEvents = 'none';
                console.log('[DEBUG] Hidden browser timeout warning as fallback');
              }
            }
          }
        } catch (e) {
          console.error('[DEBUG] Failed to dismiss browser timeout warning:', e);
        }
      }
    });
    
    // Set up observer to catch future browser error messages
    const observer = new MutationObserver((mutations) => {
      mutations.forEach((mutation) => {
        if (mutation.addedNodes && mutation.addedNodes.length > 0) {
          for (let i = 0; i < mutation.addedNodes.length; i++) {
            const node = mutation.addedNodes[i];
            if (node.nodeType === 1) { // Only process Element nodes
              const element = node as Element;
              if (element.getAttribute('role') === 'alert') {
                console.log('[DEBUG] Observer caught alert element, hiding it immediately');
                // Hide it immediately
                if (element instanceof HTMLElement) {
                  element.style.display = 'none';
                  element.style.visibility = 'hidden';
                  element.style.opacity = '0';
                  element.style.pointerEvents = 'none';
                }
                
                const text = element.textContent || '';
                if (text.includes('Loading is taking longer') || 
                    text.includes('taking longer than expected') || 
                    text.includes('Refresh')) {
                  console.log('[DEBUG] Observer caught browser timeout warning, attempting to dismiss');
                  try {
                    // Find and click the close button, typically the first button element
                    const closeButton = element.querySelector('button');
                    if (closeButton) {
                      setTimeout(() => {
                        try {
                          closeButton.click();
                          console.log('[DEBUG] Automatically dismissed browser timeout warning');
                        } catch (clickError) {
                          console.error('[DEBUG] Failed to click close button:', clickError);
                        }
                      }, 100); // Small delay to ensure the DOM is ready
                    }
                    
                    // Also try removing the element entirely
                    setTimeout(() => {
                      if (element && element.parentNode) {
                        try {
                          element.parentNode.removeChild(element);
                          console.log('[DEBUG] Removed browser timeout warning from DOM');
                        } catch (removeError) {
                          console.error('[DEBUG] Failed to remove element:', removeError);
                          // Fall back to hiding the element if removal fails
                          if (element instanceof HTMLElement) {
                            element.style.display = 'none';
                            element.style.opacity = '0';
                            element.style.pointerEvents = 'none';
                            console.log('[DEBUG] Hidden browser timeout warning as fallback');
                          }
                        }
                      }
                    }, 200);
                  } catch (e) {
                    console.error('[DEBUG] Failed to dismiss browser timeout warning:', e);
                  }
                }
              }
            }
          }
        }
      });
    });
    
    // Start observing the document with the configured parameters
    observer.observe(document.body, { childList: true, subtree: true });
    
    return observer;
  };
  
  // Initialize the error dismissal
  dismissBrowserTimeoutErrors();
};

// Add global error handling for chunk loading failures and network issues
window.addEventListener('error', (event) => {
  const errorText = event.message || 'Unknown error';
  const errorSource = event.filename || 'Unknown source';
  
  console.error('[ERROR]', errorText, errorSource, event);
  
  // Check if this is a chunk loading error or network-related
  const isChunkLoadError = errorText.includes('Loading chunk') || 
                           errorText.includes('Loading CSS chunk') ||
                           errorText.includes('Failed to fetch dynamically imported module');
                           
  const isNetworkError = errorText.includes('NetworkError') || 
                         errorText.includes('Failed to fetch') ||
                         errorText.includes('Network request failed');
                         
  if (isChunkLoadError || isNetworkError) {
    networkErrorCount++;
    console.warn(`[ERROR] Network-related error detected (${networkErrorCount}/${MAX_NETWORK_ERRORS})`);
    
    if (networkErrorCount >= MAX_NETWORK_ERRORS) {
      toast.error(
        "Problemy z połączeniem sieciowym", 
        { 
          description: "Występują problemy z ładowaniem aplikacji. Sprawdź połączenie internetowe.",
          duration: 10000
        }
      );
      
      // Reset the counter after some time
      setTimeout(() => {
        networkErrorCount = 0;
      }, NETWORK_ERROR_RESET_INTERVAL);
    }
  }
  
  // Always try to suppress browser error dialogs
  event.preventDefault();
  return true;
});

// Initialize the app
const initializeApp = async () => {
  try {
    // Apply error message blocking first
    addBrowserErrorBlockingStyles();
    
    // Add a timeout to the database initialization to prevent infinite loading
    const dbInitPromise = Promise.race([
      initializeDatabaseStructures(),
      new Promise((_, reject) => {
        setTimeout(() => {
          reject(new Error('Database initialization timed out after 10 seconds'));
        }, 10000);
      })
    ]).catch(error => {
      console.warn('[DEBUG] Database initialization had an issue, but we will continue:', error);
      // Continue even if there's an issue with DB initialization
      return null;
    });
    
    await dbInitPromise;
    
    // Setup tab visibility handling to prevent refreshes
    setupStatePreservation();
    
    ReactDOM.createRoot(document.getElementById('root')!).render(
      <React.StrictMode>
        <App />
      </React.StrictMode>
    );
    
    // Track app load time
    const loadTime = Date.now() - APP_LOAD_START_TIME;
    console.log(`[DEBUG] App initialized in ${loadTime}ms`);
    
    if (loadTime > MAX_APP_LOAD_TIME) {
      console.warn(`[PERFORMANCE] App took ${loadTime}ms to load, which exceeds the ${MAX_APP_LOAD_TIME}ms threshold`);
    }
  } catch (error) {
    console.error('[FATAL] Failed to initialize app:', error);
    // Show a fatal error message
    document.getElementById('root')!.innerHTML = `
      <div style="padding: 20px; text-align: center; font-family: system-ui, sans-serif;">
        <h2>Nie można uruchomić aplikacji</h2>
        <p>Wystąpił krytyczny błąd. Odśwież stronę lub spróbuj ponownie później.</p>
        <button onclick="window.location.reload()" style="padding: 8px 16px; margin-top: 16px; cursor: pointer;">
          Odśwież stronę
        </button>
      </div>
    `;
  }
};

initializeApp();
